#include "Rect.h"

// Public
void Rect::Update( int moveX, int moveY, int growX, int growY )
{
	if (CanMove( moveX, Graphics::ScreenWidth ))
	{
		left += moveX;
		right += moveX;
	}

	if (CanMove( moveY, Graphics::ScreenHeight ))
	{
		top += moveY;
		bottom += moveY;
	}

	if (CanGrow( growX, Graphics::ScreenWidth ))
		right += growX;

	if (CanGrow( growY, Graphics::ScreenHeight ))
		bottom += growY;
}

void Rect::Draw( Graphics & gfx ) const
{
	const int xStart = max( left, 0 );
	const int xEnd = min( right, Graphics::ScreenWidth );
	for (int i = xStart; i < xEnd; ++i)
	{
		const int yStart = max( top, 0 );
		const int yEnd = min( bottom, Graphics::ScreenHeight );
		for (int j = yStart; j < yEnd; ++j)
		{
			gfx.PutPixel( i, j, 255, 255, 255 );
		}
	}
}

// Private
bool Rect::CanMove( int amount, int screenEdge )
{
	int topLeft, bottomRight;
	if (screenEdge == Graphics::ScreenWidth)
	{
		topLeft = left + minWidth;
		bottomRight = right - minWidth;
	}
	else if (screenEdge == Graphics::ScreenHeight)
	{
		topLeft = top + minHeight;
		bottomRight = bottom - minHeight;
	}
	else
	{
		topLeft += screenEdge;
	}

	topLeft += amount;
	bottomRight += amount;

	return topLeft <= screenEdge
		&& bottomRight >= 0;
}

bool Rect::CanGrow( int growAmount, int screenEdge )
{
	int movingEdge, otherEdge, minimum;
	if (screenEdge == Graphics::ScreenWidth)
	{
		movingEdge = right;
		otherEdge = left;
		minimum = minWidth;
	}
	else if (screenEdge == Graphics::ScreenHeight)
	{
		movingEdge = bottom;
		otherEdge = top;
		minimum = minHeight;
	}
	else
	{
		minimum = -1;
	}

	return minimum > 0
		&& movingEdge + growAmount >= minimum
		&& movingEdge + growAmount - otherEdge >= minimum;
}
